<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use URL;
use App\UserDocument, App\User, App\Transaction, App\UserDepositAccount, App\SendingCompany, App\DepositCompany, App\TempTransaction, App\Country, App\UserTransaction, App\UserTransactionStatus;
use Session;
use Auth;
use DB;
use Hash;
use Illuminate\Support\Facades\Input;
use File;
use Illuminate\Support\Facades\Mail;

use App\Libs\sending_companies_api\Gremitlib;

class UserController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Contracts\Support\Renderable
     */
    public function index()
    {

    }

    public function verification()
    {
        $user_info = User::find(Auth::id());
        
        $user_document = UserDocument::where('user_id', '=', Auth::user()->id)->first();

        return view('verification', ["user_info" => $user_info, "user_document" => $user_document]);
    }

    public function verification_post(Request $request)
    {
        $rules = [
            'verification_doc' => 'required',
            'profile_image' => 'required',
            'verification_doc' => 'max:2000',
            'profile_image' => 'max:2000',
            'agree' => ['required', 'in:1'],
        ];

        $this->validate($request, $rules, ['verification_doc.required' => 'Please upload a verification document.', 'profile_image.required' => 'Please upload a profile image.', 'verification_doc.max' => 'The verification document max file size should be 2MB.', 'profile_image.max' => 'The profile image max file size should be 2MB.']);

        $url = '';

        $verification_doc_attached = false;
        $profile_image_attached = false;

        if($request->hasFile('verification_doc') && 
            $request->file('verification_doc'))
        {
            $verification_doc_attached = true;

            // Start upload verification doc

            $file = $request->file('verification_doc');
            $filenameWithExtension = $request->file('verification_doc')->getClientOriginalName();
            $filename = pathinfo($filenameWithExtension, PATHINFO_FILENAME);
            $extension = $request->file('verification_doc')->getClientOriginalExtension();
            $filenameToStore = time()."_".$filename.'.'.$extension;

            $folder_to_upload = public_path().'/public/uploads/user_documents/';

            if (!File::exists($folder_to_upload)) {
                File::makeDirectory($folder_to_upload, 0777, true, true);
            }

            $file->move("public/uploads/user_documents/", $filenameToStore);
            $url = URL::to('/')."/public/uploads/user_documents/".$filenameToStore;

            $user_document = UserDocument::where('user_id', '=', Auth::user()->id)->first();

            if(!empty($user_document) && !empty($user_document->toArray()))
            {
                UserDocument::find($user_document->id)->delete();

                $file_path = public_path().'/public/uploads/user_documents/'.$user_document->document;

                unlink($file_path);
            }

            $crud_data_user_document = array(
                'user_id' => Auth::id(),
                'document' => $filenameToStore,
            );

            $res = UserDocument::create($crud_data_user_document);

            // End upload verification doc
        }
        else
        {
            return redirect()->route('verification')->with('error', 'Please select your documents.');
        }

        if($verification_doc_attached == true)
        {
            if(!empty($res) && !empty($res->toArray()) )
            {
                $user_info = User::find(Auth::id());
                $user_full_name = $user_info->first_name." ".$user_info->last_name;
                $user_email = $user_info->email;
                $user_phone_number = $user_info->phone_number;
                if(isset($user_info->country))
                {
                    $user_country = $user_info->country->country_name;
                }
                else
                {
                    $user_country = '';
                }

                $admin_user = DB::table('users')->where('user_role_id', config('constants.user_roles.ADMIN_ROLE_ID'))->first();

                $admin_full_name = $admin_user->first_name." ".$admin_user->last_name;
                $admin_email = $admin_user->email;

                $doc_upload_email = array(
                    'email_type' => 'USER_UPLOAD_DOCUMENT',

                    'var_greeting_name' => $admin_full_name,
                    'email' => $admin_email,

                    'user_full_name' => $user_full_name,
                    'user_email' => $user_email,
                    'user_phone_number' => $user_phone_number,
                    'user_country' => $user_country
                );

                Mail::send('emails.mail', $doc_upload_email, function($message) use ($admin_email) {
                    $message->to($admin_email)->subject('User uploaded documents to BedouApp');
                });

                // return redirect('/receiver-info')->with('success', 'Thank you for uploading user verification documents.');
                // return redirect()->route('receiver-info')->with('success', 'Thank you for uploading user verification documents and user profile image.');
            }
            else
            {
                return redirect()->route('verification')->with('error', 'Failed to upload user verification documents or user profile image.');
            }
        }
        
        
        if($request->hasFile('profile_image') && 
            $request->file('profile_image'))
        {
            $profile_image_attached = true;

            // Start upload profile image

            $file = $request->file('profile_image');
            $filenameWithExtension = $request->file('profile_image')->getClientOriginalName();
            $filename = pathinfo($filenameWithExtension, PATHINFO_FILENAME);
            $extension = $request->file('profile_image')->getClientOriginalExtension();
            $filenameToStore = time()."_".$filename.'.'.$extension;

            $folder_to_upload = public_path().'/public/uploads/user_profile_image/';

            if (!File::exists($folder_to_upload)) {
                File::makeDirectory($folder_to_upload, 0777, true, true);
            }

            $file->move("public/uploads/user_profile_image/", $filenameToStore);
            $url = URL::to('/')."/public/uploads/user_profile_image/".$filenameToStore;

            $crud_data_user = array(
                'profile_image' => $filenameToStore,
            );

            $res_user_update = DB::table('users')->where('id', Auth::id())->update($crud_data_user);

            // End upload profile image
        }
        else
        {
            return redirect()->route('verification')->with('error', 'Please select your documents.');
        }

        if($profile_image_attached == true)
        {
            if($res_user_update)
            {
                // return redirect('/receiver-info')->with('success', 'Thank you for uploading user verification documents.');
                // return redirect()->route('receiver-info')->with('success', 'Thank you for uploading user verification documents and user profile image.');
            }
            else
            {
                return redirect()->route('verification')->with('error', 'Failed to upload user verification documents or user profile image.');
            }
        }

        _set_flashdata('S', 'Thank you for uploading user verification document.', 'FLASH_HTML');

        return redirect()->route('home');
        // return redirect()->route('home')->with('success', 'Thank you for uploading user verification document.');
    }

    public function my_account()
    {
        $user = User::find(Auth::user()->id);
        // dd($user->country);

        $user_document = UserDocument::where('user_id', '=', Auth::user()->id)->first();

        return view('my_account', ["user" => $user, "user_document" => $user_document]);
    }

    public function change_password()
    {
        return view('change_password');
    }

    public function change_password_post(Request $request)
    {
        $this->validate(request(), [
            'current_password' => 'required',
            'password' => 'required|string|confirmed|min:7|regex:/^(?=.*?[A-Z])(?=.*?[a-z]).{7,}$/',
            'password_confirmation' => 'required|same:password',
        ],[
            'password.regex' => 'Password is invalid. (Should have minimum 7 characters with atleast 1 capital letter, 1 small letter)',
        ]);

        $user_info = User::where('id', '=', Auth::user()->id)->first();

        $current_password_db = $user_info->password;

        $current_password = $request->current_password;

        if(!Hash::check($current_password, $current_password_db))
        {
            return redirect('/change-password')->with('error', 'Your current password does not match. Please enter correct one.');
        }

        $password = $request->password;

        if(Hash::check($password, $current_password_db))
        {
            return redirect('/change-password')->with('error', 'Your new password is same as your current password.');
        }

        $crud_data = array(
            'password' => Hash::make($request->password),
        );

        $result = DB::table('users')->where('id', Auth::user()->id)->update($crud_data);

        if($result)
        {
            return redirect('/my-account')->with('success', 'Congratulation!! Your password changed successfully.');
        }
        else
        {
            return redirect('/my-account')->with('error', 'Oops, something went wrong.<br>Looks like we\'re a little hung up. <br>We\'re working on it - check back soon!<br>[Error Code:2002]');
        }        
    }

    public function my_account_form()
    {
        $user = User::find(Auth::user()->id);

        $countries = Country::get();

        return view('my_account_form', ["user" => $user, "countries" => $countries]);
    }

    public function my_account_form_post(Request $request)
    {
        $this->validate(request(), [
            'phone_number' => ['required','numeric'],
            'address' => ['required'],
            'city' => ['required'],
            'country_id' => ['required'],
        ]);

        $phone_number = $request->phone_number;
        $address = $request->address;
        $city = $request->city;
        $country_id = $request->country_id;

        $crud_data = array(
            'phone_number' => $phone_number,
            'address' => $address,
            'city' => $city,
            'country_id' => $country_id,
        );

        $result = DB::table('users')->where('id', Auth::user()->id)->update($crud_data);

        if($result)
        {
            $back_url = session()->get('back-url');
            if($back_url == "home")
            {
                session()->forget('back-url');
                _set_flashdata('S', 'Now you can create transactions process.', 'FLASH_HTML');
                return redirect()->route('home');
            }
            else
            {
                return redirect('/my-account')->with('success', 'Congratulation!! Your account information updated successfully.');
            }
        }
        else
        {
            return redirect('/my-account')->with('error', 'Oops, something went wrong.<br>Looks like we\'re a little hung up. <br>We\'re working on it - check back soon!<br>[Error Code:2002]');
        }
    }
}
